@@ -1,4 +1,3 @@
-# $Header: /home/jesse/DBIx-SearchBuilder/history/SearchBuilder/Handle.pm,v 1.21 2002/01/28 06:11:37 jesse Exp $
 package DBIx::SearchBuilder::Handle;
 
 use strict;
@@ -8,6 +7,7 @@ use Carp qw(croak cluck);
 use DBI;
 use Class::ReturnValue;
 use Encode qw();
+use version;
 
 use DBIx::SearchBuilder::Util qw/ sorted_values /;
 
@@ -28,12 +28,12 @@ DBIx::SearchBuilder::Handle - Perl extension which is a generic DBI handle
                     Host => 'hostname',
                     User => 'dbuser',
                     Password => 'dbpassword');
-  # now $handle isa DBIx::SearchBuilder::Handle::mysql                    
- 
+  # now $handle isa DBIx::SearchBuilder::Handle::mysql
+
 =head1 DESCRIPTION
 
 This class provides a wrapper for DBI handles that can also perform a number of additional functions.
- 
+
 =cut
 
 
@@ -62,15 +62,15 @@ sub new  {
 
 =head2 Connect PARAMHASH: Driver, Database, Host, User, Password, QuoteTableNames
 
-Takes a paramhash and connects to your DBI datasource. 
+Takes a paramhash and connects to your DBI datasource.
 
 You should _always_ set
 
-     DisconnectHandleOnDestroy => 1 
+     DisconnectHandleOnDestroy => 1
 
 unless you have a legacy app like RT2 or RT 3.0.{0,1,2} that depends on the broken behaviour.
 
-If you created the handle with 
+If you created the handle with
      DBIx::SearchBuilder::Handle->new
 and there is a DBIx::SearchBuilder::Handle::(Driver) subclass for the driver you have chosen,
 the handle will be automatically "upgraded" into that subclass.
@@ -143,7 +143,7 @@ of the standard driver-specific subclasses.
 
 sub _UpgradeHandle {
     my $self = shift;
-    
+
     my $driver = shift;
     my $class = 'DBIx::SearchBuilder::Handle::' . $driver;
     local $@;
@@ -157,7 +157,7 @@ sub _UpgradeHandle {
 
 =head2 BuildDSN PARAMHASH
 
-Takes a bunch of parameters:  
+Takes a bunch of parameters:
 
 Required: Driver, Database,
 Optional: Host, Port and RequireSSL
@@ -209,7 +209,7 @@ Turns on the Database Handle's RaiseError attribute.
 sub RaiseError {
     my $self = shift;
 
-    my $mode = 1; 
+    my $mode = 1;
     $mode = shift if (@_);
 
     $self->dbh->{RaiseError}=$mode;
@@ -227,7 +227,7 @@ Turns on the Database Handle's PrintError attribute.
 sub PrintError {
     my $self = shift;
 
-    my $mode = 1; 
+    my $mode = 1;
     $mode = shift if (@_);
 
     $self->dbh->{PrintError}=$mode;
@@ -270,7 +270,7 @@ sub _LogSQLStatement {
 
 =head2 ClearSQLStatementLog
 
-Clears out the SQL statement log. 
+Clears out the SQL statement log.
 
 
 =cut
@@ -278,12 +278,12 @@ Clears out the SQL statement log.
 sub ClearSQLStatementLog {
     my $self = shift;
     @{$self->{'StatementLog'}} = ();
-}   
+}
 
 
 =head2 SQLStatementLog
 
-Returns the current SQL statement log as an array of arrays. Each entry is a triple of 
+Returns the current SQL statement log as an array of arrays. Each entry is a triple of
 
 (Time,  Statement, Duration)
 
@@ -306,7 +306,7 @@ Turns on the Database Handle's AutoCommit attribute.
 sub AutoCommit {
     my $self = shift;
 
-    my $mode = 1; 
+    my $mode = 1;
     $mode = shift if (@_);
 
     $self->dbh->{AutoCommit}=$mode;
@@ -333,10 +333,10 @@ sub Disconnect  {
     # interacting with a disconnected handle, here we unset
     # dbh to inform other code that there is no connection any more.
     # See also https://github.com/perl5-dbi/DBD-mysql/issues/306
-
+    my ($version) = ( $self->DatabaseVersion // '' ) =~ /^(\d+\.\d+)/;
     if (   $self->isa('DBIx::SearchBuilder::Handle::mysql')
         && $self->{'database_version'} =~ /mariadb/i
-        && $self->{'database_version'} ge '10.2' )
+        && version->parse('v'.$version) > version->parse('v10.2') )
     {
         $self->dbh(undef);
     }
@@ -356,7 +356,7 @@ Return the current DBI handle. If we're handed a parameter, make the database ha
 
 sub dbh {
   my $self=shift;
-  
+
   #If we are setting the database handle, set it.
   if ( @_ ) {
       $DBIHandle{$self} = $PrevHandle = shift;
@@ -441,15 +441,15 @@ sub InsertFromSelect {
     return $rows == 0? '0E0' : $rows;
 }
 
-=head2 UpdateRecordValue 
+=head2 UpdateRecordValue
 
-Takes a hash with fields: Table, Column, Value PrimaryKeys, and 
-IsSQLFunction.  Table, and Column should be obvious, Value is where you 
-set the new value you want the column to have. The primary_keys field should 
-be the lvalue of DBIx::SearchBuilder::Record::PrimaryKeys().  Finally 
-IsSQLFunction is set when the Value is a SQL function.  For example, you 
-might have ('Value'=>'PASSWORD(string)'), by setting IsSQLFunction that 
-string will be inserted into the query directly rather then as a binding. 
+Takes a hash with fields: Table, Column, Value PrimaryKeys, and
+IsSQLFunction.  Table, and Column should be obvious, Value is where you
+set the new value you want the column to have. The primary_keys field should
+be the lvalue of DBIx::SearchBuilder::Record::PrimaryKeys().  Finally
+IsSQLFunction is set when the Value is a SQL function.  For example, you
+might have ('Value'=>'PASSWORD(string)'), by setting IsSQLFunction that
+string will be inserted into the query directly rather then as a binding.
 
 =cut
 
@@ -466,25 +466,25 @@ sub UpdateRecordValue {
     my $query = 'UPDATE ' . $args{'Table'} . ' ';
      $query .= 'SET '    . $args{'Column'} . '=';
 
-  ## Look and see if the field is being updated via a SQL function. 
-  if ($args{'IsSQLFunction'}) {
-     $query .= $args{'Value'} . ' ';
-  }
-  else {
-     $query .= '? ';
-     push (@bind, $args{'Value'});
-  }
+    ## Look and see if the field is being updated via a SQL function.
+    if ($args{'IsSQLFunction'}) {
+       $query .= $args{'Value'} . ' ';
+    }
+    else {
+       $query .= '? ';
+       push (@bind, $args{'Value'});
+    }
 
-  ## Constructs the where clause.
-  my $where  = 'WHERE ';
-  foreach my $key (sort keys %{$args{'PrimaryKeys'}}) {
-     $where .= $key . "=?" . " AND ";
-     push (@bind, $args{'PrimaryKeys'}{$key});
-  }
-     $where =~ s/AND\s$//;
-  
-  my $query_str = $query . $where;
-  return ($self->SimpleQuery($query_str, @bind));
+    ## Constructs the where clause.
+    my $where  = 'WHERE ';
+    foreach my $key (sort keys %{$args{'PrimaryKeys'}}) {
+       $where .= $key . "=?" . " AND ";
+       push (@bind, $args{'PrimaryKeys'}{$key});
+    }
+    $where =~ s/AND\s$//;
+
+    my $query_str = $query . $where;
+    return ($self->SimpleQuery($query_str, @bind));
 }
 
 
@@ -500,12 +500,12 @@ don\'t quote the NEW_VALUE
 sub UpdateTableValue  {
     my $self = shift;
 
-    ## This is just a wrapper to UpdateRecordValue().     
-    my %args = (); 
+    ## This is just a wrapper to UpdateRecordValue().
+    my %args = ();
     $args{'Table'}  = shift;
     $args{'Column'} = shift;
     $args{'Value'}  = shift;
-    $args{'PrimaryKeys'}   = shift; 
+    $args{'PrimaryKeys'}   = shift;
     $args{'IsSQLFunction'} = shift;
 
     return $self->UpdateRecordValue(%args)
@@ -602,13 +602,25 @@ sub SimpleQuery {
         }
     }
 
-    # Check @bind_values for HASH refs
+    # Check @bind_values for HASH refs. These can be sent as type settings
+    # or hits for the DBD module for dealing with certain data types,
+    # usually BLOBS.
     for ( my $bind_idx = 0 ; $bind_idx < scalar @bind_values ; $bind_idx++ ) {
         if ( ref( $bind_values[$bind_idx] ) eq "HASH" ) {
             my $bhash = $bind_values[$bind_idx];
             $bind_values[$bind_idx] = $bhash->{'value'};
             delete $bhash->{'value'};
-            $sth->bind_param( $bind_idx + 1, undef, $bhash );
+
+            # DBD::MariaDB only works using the string version of setting type,
+            # so convert the hash to the string for this case.
+            if ( $self->isa('DBIx::SearchBuilder::Handle::MariaDB')
+                 && $bhash->{'TYPE'}
+                 && $bhash->{'TYPE'} eq 'SQL_BLOB' ) {
+                $sth->bind_param( $bind_idx + 1, undef, DBI::SQL_BLOB );
+            }
+            else {
+                $sth->bind_param( $bind_idx + 1, undef, $bhash );
+            }
         }
     }
 
@@ -658,19 +670,19 @@ If the select succeeds, returns the first row as an array.
 Otherwise, returns a Class::ResturnValue object with the failure loaded
 up.
 
-=cut 
+=cut
 
 sub FetchResult {
-  my $self = shift;
-  my $query = shift;
-  my @bind_values = @_;
-  my $sth = $self->SimpleQuery($query, @bind_values);
-  if ($sth) {
-    return ($sth->fetchrow);
-  }
-  else {
-   return($sth);
-  }
+    my $self = shift;
+    my $query = shift;
+    my @bind_values = @_;
+    my $sth = $self->SimpleQuery($query, @bind_values);
+    if ($sth) {
+        return ($sth->fetchrow);
+    }
+    else {
+        return($sth);
+    }
 }
 
 
@@ -704,14 +716,14 @@ sub KnowsBLOBs {
 
 =head2 BLOBParams FIELD_NAME FIELD_TYPE
 
-Returns a hash ref for the bind_param call to identify BLOB types used by 
-the current database for a particular column type.                 
+Returns a hash ref for the bind_param call to identify BLOB types used by
+the current database for a particular column type.
 
 =cut
 
 sub BLOBParams {
     my $self = shift;
-    # Don't assign to key 'value' as it is defined later. 
+    # Don't assign to key 'value' as it is defined later.
     return ( {} );
 }
 
@@ -983,19 +995,25 @@ sub ApplyLimits {
     my $statementref = shift;
     my $per_page = shift;
     my $first = shift;
+    my $sb = shift;
 
     my $limit_clause = '';
 
     if ( $per_page) {
         $limit_clause = " LIMIT ";
+        if ( $sb->{_bind_values} ) {
+            push @{$sb->{_bind_values}}, $first || (), $per_page;
+            $first = '?' if $first;
+            $per_page = '?';
+        }
+
         if ( $first ) {
             $limit_clause .= $first . ", ";
         }
         $limit_clause .= $per_page;
     }
 
-   $$statementref .= $limit_clause; 
-
+    $$statementref .= $limit_clause;
 }
 
 
@@ -1004,8 +1022,8 @@ sub ApplyLimits {
 
 =head2 Join { Paramhash }
 
-Takes a paramhash of everything Searchbuildler::Record does 
-plus a parameter called 'SearchBuilder' that contains a ref 
+Takes a paramhash of everything Searchbuildler::Record does
+plus a parameter called 'SearchBuilder' that contains a ref
 to a SearchBuilder object'.
 
 This performs the join.
@@ -1216,7 +1234,7 @@ sub _NormalJoin {
     }
 }
 
-# this code is all hacky and evil. but people desperately want _something_ and I'm 
+# this code is all hacky and evil. but people desperately want _something_ and I'm
 # super tired. refactoring gratefully appreciated.
 
 sub _BuildJoins {
@@ -1341,7 +1359,7 @@ sub MayBeNull {
 
     # replace conditions with boolean result: 1 - allows nulls, 0 - not
     # all restrictions on that don't act on required alias allow nulls
-    # otherwise only IS NULL operator 
+    # otherwise only IS NULL operator
     foreach ( splice @conditions ) {
         unless ( ref $_ ) {
             push @conditions, $_;
@@ -1436,7 +1454,7 @@ sub MayBeNull {
     return 1;
 }
 
-=head2 DistinctQuery STATEMENTREF 
+=head2 DistinctQuery STATEMENTREF
 
 takes an incomplete SQL SELECT statement and massages it to return a DISTINCT result set.
 
@@ -1447,6 +1465,10 @@ sub DistinctQuery {
     my $self = shift;
     my $statementref = shift;
     my $sb = shift;
+    my %args = (
+        Wrap => 0,
+        @_
+    );
 
     my $QueryHint = $sb->QueryHint;
     $QueryHint = $QueryHint ? " /* $QueryHint */ " : " ";
@@ -1454,13 +1476,44 @@ sub DistinctQuery {
     # Prepend select query for DBs which allow DISTINCT on all column types.
     $$statementref = "SELECT" . $QueryHint . "DISTINCT main.* FROM $$statementref";
     $$statementref .= $sb->_GroupClause;
+    if ( $args{'Wrap'} ) {
+        $$statementref = "SELECT * FROM ($$statementref) main";
+    }
     $$statementref .= $sb->_OrderClause;
 }
 
+=head2 DistinctQueryAndCount STATEMENTREF
+
+takes an incomplete SQL SELECT statement and massages it to return a
+DISTINCT result set and the total count of potential records.
+
+=cut
+
+sub DistinctQueryAndCount {
+    my $self = shift;
+    my $statementref = shift;
+    my $sb = shift;
+
+    # order by clause should be on outer most query
+    # SQL standard: ORDER BY command cannot be used in a Subquery
+    # mariadb explanation: https://mariadb.com/kb/en/why-is-order-by-in-a-from-subquery-ignored/
+    # pg: actually keeps order of a subquery as long as some conditions in outer query are met, but
+    #     it's just a coincidence, not a feature
+    # SQL server: https://learn.microsoft.com/en-us/sql/t-sql/queries/select-order-by-clause-transact-sql?view=sql-server-ver16
+    #             The ORDER BY clause is not valid in views, ... and *subqueries*, unless ...
+
+    my $order = $sb->_OrderClause;
+    my $wrap = $order !~ /(?<!main)\./;
+
+    $self->DistinctQuery($statementref, $sb, Wrap => $wrap);
+
+    # DistinctQuery already has an outer SELECT, which we can reuse
+    $$statementref =~ s!(?= FROM)!, COUNT(main.id) OVER() AS search_builder_count_all!;
+}
+
 
 
-
-=head2 DistinctCount STATEMENTREF 
+=head2 DistinctCount STATEMENTREF
 
 takes an incomplete SQL SELECT statement and massages it to return a DISTINCT result set.
 
@@ -1507,10 +1560,9 @@ Currently prints that message to STDERR
 =cut
 
 sub Log {
-	my $self = shift;
-	my $msg = shift;
-	warn $msg."\n";
-
+    my $self = shift;
+    my $msg = shift;
+    warn $msg."\n";
 }
 
 =head2 SimpleDateTimeFunctions
@@ -1770,6 +1822,16 @@ sub HasSupportForNullsOrder {
     return 0;
 }
 
+=head2 HasSupportForCombineSearchAndCount
+
+Returns true value if DB supports to combine search and count in single
+query.
+
+=cut
+
+sub HasSupportForCombineSearchAndCount {
+    return 1;
+}
 
 =head2 QuoteName
 
@@ -1813,6 +1875,77 @@ sub DequoteName {
     return $name;
 }
 
+sub _ExtractBindValues {
+    my $self                = shift;
+    my $string              = shift;
+    my $default_escape_char = shift || q{'};
+    return $string unless defined $string;
+
+    my $placeholder = '';
+
+    my @chars       = split //, $string;
+    my $value       = '';
+    my $escape_char = $default_escape_char;
+
+    my @values;
+    my $in = 0;    # keep state in the loop: is it in a quote?
+    while ( defined( my $c = shift @chars ) ) {
+        my $escaped;
+        if ( $c eq $escape_char && $in ) {
+            if ( $escape_char eq q{'} ) {
+                if ( ( $chars[0] || '' ) eq q{'} ) {
+                    $c       = shift @chars;
+                    $escaped = 1;
+                }
+            }
+            else {
+                $c       = shift @chars;
+                $escaped = 1;
+            }
+        }
+
+        if ($in) {
+            if ( $c eq q{'} ) {
+                if ( !$escaped ) {
+                    push @values, $value;
+                    $in          = 0;
+                    $value       = '';
+                    $escape_char = $default_escape_char;
+                    $placeholder .= '?';
+                    next;
+                }
+            }
+            $value .= $c;
+        }
+        else {
+            if ( $c eq q{'} ) {
+                $in = 1;
+            }
+
+            # Handle quoted string like e'foo\\bar'
+            elsif ( lc $c eq 'e' && ( $chars[0] // '' ) eq q{'} ) {
+                $escape_char = '\\';
+            }
+
+            # Handle numbers
+            elsif ( $c =~ /[\d.]/ && $placeholder !~ /\w$/ ) {    # Do not catch Groups_1.Name
+                $value .= $c;
+                while ( ( $chars[0] // '' ) =~ /[\d.]/ ) {
+                    $value .= shift @chars;
+                }
+
+                push @values, $value;
+                $placeholder .= '?';
+                $value = '';
+            }
+            else {
+                $placeholder .= $c;
+            }
+        }
+    }
+    return ( $placeholder, @values );
+}
+
 sub _RequireQuotedTables { return 0 };
 
 =head2 DESTROY
@@ -1822,23 +1955,23 @@ When we get rid of the Searchbuilder::Handle, we need to disconnect from the dat
 =cut
 
 sub DESTROY {
-  my $self = shift;
-  $self->Disconnect if $self->{'DisconnectHandleOnDestroy'};
-  delete $DBIHandle{$self};
+    my $self = shift;
+    $self->Disconnect if $self->{'DisconnectHandleOnDestroy'};
+    delete $DBIHandle{$self};
 }
 
+=head2 CastAsDecimal FIELD
 
-1;
-__END__
+Cast the given field as decimal.
 
-
-=head1 AUTHOR
-
-Jesse Vincent, jesse@fsck.com
-
-=head1 SEE ALSO
-
-perl(1), L<DBIx::SearchBuilder>
+E.g. on Pg, it's C<CAST(Content AS DECIMAL)>.
 
 =cut
 
+sub CastAsDecimal {
+    my $self  = shift;
+    my $field = shift or return;
+    return "CAST($field AS DECIMAL)";
+}
+
+1;
