@@ -853,7 +853,7 @@ sub __Set {
         # Support for databases which don't deal with LOBs automatically
         my $ca = $self->_ClassAccessible();
         my $key = $args{'Column'};
-        if ( $ca->{$key}->{'type'} =~ /^(text|longtext|clob|blob|lob)$/i ) {
+        if ( ( $ca->{$key}->{'type'} // '' ) =~ /^(text|longtext|clob|longblob|blob|lob)$/i ) {
             my $bhash = $self->_Handle->BLOBParams( $key, $ca->{$key}->{'type'} );
             $bhash->{'value'} = $args{'Value'};
             $args{'Value'} = $bhash;
@@ -1302,11 +1302,13 @@ sub Create {
         my $ca = $self->_ClassAccessible();
         foreach $key ( keys %attribs ) {
             my $type = $ca->{$key}->{'type'};
-            next unless $type && $type =~ /^(text|longtext|clob|blob|lob)$/i;
+            next unless $type && $type =~ /^(text|longtext|clob|blob|lob|longblob)$/i;
 
             my $bhash = $self->_Handle->BLOBParams( $key, $type );
-            $bhash->{'value'} = $attribs{$key};
-            $attribs{$key} = $bhash;
+            if ( ref($bhash) eq 'HASH' ) {
+                $bhash->{'value'} = $attribs{$key};
+                $attribs{$key} = $bhash;
+            }
         }
     }
     return ( $self->_Handle->Insert( $self->Table, %attribs ) );
